//#include "EventSink.h"

#include <iostream>
using namespace std;
#include <comdef.h>
#include "cEventSink.h"

#pragma comment(lib, "wbemuuid.lib")

extern "C" IMyEventSink *CreateInstance();

int main(int iArgcCnt, char** argv)
{
	HRESULT hres;

	/* 1] init COM */
	hres = CoInitializeEx(0, COINITBASE_MULTITHREADED);
	if (FAILED(hres))
	{
		cout << "Failed to init COM. Erro code = 0x" << hex << hres << endl;
		return 1;
	}

	/* 2] set general COM security levels */
	hres = CoInitializeSecurity(
		NULL,
		-1,				// COM negotiates service
		NULL,			// Authentication services
		NULL,			// Reserved
		RPC_C_AUTHN_LEVEL_DEFAULT,		// Default authentication
		RPC_C_IMP_LEVEL_IMPERSONATE,	// Default impersonation
		NULL,							// Authentication info
		EOAC_NONE,						// Additional capabilities
		NULL							// Reserved		
		);

	if (FAILED(hres))
	{
		cout << "Failed to init security. Erro code = 0x" << hex << hres << endl;
		CoUninitialize();
		return 1;
	}

	/* 3] obtain the initial locator to WMI */
	IWbemLocator *pLoc = NULL;

	hres = CoCreateInstance(
		CLSID_WbemLocator,
		0,
		CLSCTX_INPROC_SERVER,
		IID_IWbemLocator,
		(LPVOID *)&pLoc
		);

	if (FAILED(hres))
	{
		cout << "Failed to create IWbemLocator object. " << "Err code = 0x" << hex << hres << endl;
		CoUninitialize();
		return 1;
	}

	/* 4] connect to WMI through the IWbemLocator::ConnectServer method */
	IWbemServices *pSvc = NULL;

	/* connect to the local root\cimv2 namespace and obtain pointer */
	hres = pLoc->ConnectServer(
		_bstr_t(L"ROOT\\CIMV2"),
		NULL,
		NULL,
		0,
		NULL,
		0,
		0,
		&pSvc
	);

	if (FAILED(hres))
	{
		cout << "Could not connect. Error code = 0x" << hex << hres << endl;
		pLoc->Release();
		CoUninitialize();
		return 1;
	}

	cout << "Connected to ROOT\\CIMV2 WMI namespace" << endl;

	/* 5] set security levels on the proxy */
	hres = CoSetProxyBlanket(
		pSvc,				// indicates the proxy to set
		RPC_C_AUTHN_WINNT,
		RPC_C_AUTHZ_NONE,
		NULL,				// server principal name
		RPC_C_AUTHN_LEVEL_CALL,	//
		RPC_C_IMP_LEVEL_IMPERSONATE,
		NULL,				// client identity
		EOAC_NONE			// proxy capabilities
	);

	if (FAILED(hres))
	{
		cout << "Could not set proxy blanket. Error code = 0x" << hex << hres << endl;
		pSvc->Release();
		pLoc->Release();
	}

	/* 6] receive event notification */
	IUnsecuredApartment* pUnsecApp = NULL;

	hres = CoCreateInstance(CLSID_UnsecuredApartment, NULL,	CLSCTX_LOCAL_SERVER, IID_IUnsecuredApartment, (void**)&pUnsecApp);

	IMyEventSink *pMySink = CreateInstance();
	IUnknown* pStubUnkC = NULL;
	pUnsecApp->CreateObjectStub((IUnknown*)pMySink, &pStubUnkC);
	
	IWbemObjectSink* pStubSinkC = NULL;
	pStubUnkC->QueryInterface(IID_IWbemObjectSink, (void**)&pStubSinkC);

	//__InstanceDeletionEvent __InstanceCreationEvent
	hres = pSvc->ExecNotificationQueryAsync(
		_bstr_t("WQL"),
		_bstr_t("SELECT * "
		"FROM  __InstanceDeletionEvent WITHIN 1 "
		"WHERE TargetInstance ISA 'WIn32_Process'"),
		WBEM_FLAG_SEND_STATUS,
		NULL,
		pStubSinkC
		);

	//__InstanceDeletionEvent __InstanceCreationEvent
	hres = pSvc->ExecNotificationQueryAsync(
		_bstr_t("WQL"),
		_bstr_t("SELECT * "
		"FROM  __InstanceCreationEvent WITHIN 1 "
		"WHERE TargetInstance ISA 'WIn32_Process'"),
		WBEM_FLAG_SEND_STATUS,
		NULL,
		pStubSinkC
		);


	Sleep(100000);

	hres = pSvc->CancelAsyncCall(pStubSinkC);

	/* cleanup */
	pSvc->Release();
	pLoc->Release();
	pUnsecApp->Release();
	pStubUnkC->Release();
	pStubSinkC->Release();
	pMySink->lpVtbl->Release(pMySink);

	CoUninitialize();

	return 0;
}
