#pragma once

#include <windows.h>
#include <objbase.h>
#include "cEventSink.h"


static HRESULT STDMETHODCALLTYPE QueryInterface(IMyEventSink *this, REFIID vTableGuid, void **ppv)
{
	// Check if the GUID matches IMyEventSinkExample VTable's GUID
	if (!IsEqualIID(vTableGuid, &IID_IUnknown) && !IsEqualIID(vTableGuid, &IID_IWbemObjectSink))
	{
		*ppv = 0;
		return(E_NOINTERFACE);
	}
	
	*ppv = this;

	// increment the count of callers
	this->lpVtbl->AddRef(this);

	return(NOERROR);
}


static ULONG STDMETHODCALLTYPE AddRef(IMyEventSink *this)
{
	return (++this->count);
}

static ULONG STDMETHODCALLTYPE Release(IMyEventSink *this)
{
	--this->count;

	if (this->count < 0)
		this->count = 0;

	return this->count;
}

static HRESULT STDMETHODCALLTYPE Indicate(IMyEventSink *this, LONG lObjectCount, IWbemClassObject **apObjArray)
{
	HRESULT hr = S_OK;
	VARIANT vtProp;
	WCHAR buf[128] = { 0 };
	
	VariantInit(&vtProp);
	vtProp.vt = VT_VARIANT;
	const IID const WEBCLASSIID = IID_IWbemClassObject;

	for (int i = 0; i < lObjectCount; i++)
	{
		wprintf(L"event occured\n");

		hr = apObjArray[i]->lpVtbl->Get(apObjArray[i], L"TargetInstance", 0, &vtProp, 0, 0);

		if (!FAILED(hr))
		{
			IUnknown* str = vtProp.punkVal;
			hr = str->lpVtbl->QueryInterface(str, &IID_IWbemClassObject, (void**)(&apObjArray[i]));
			if (SUCCEEDED(hr))
			{
				VARIANT cn;
				VariantInit(&vtProp);

				hr = apObjArray[i]->lpVtbl->Get(apObjArray[i], L"Name", 0, &cn, NULL, NULL);
				if (SUCCEEDED(hr))
				{
					if ((cn.vt == VT_NULL) || (cn.vt == VT_EMPTY))
					{												
						wprintf(L"Unknown process\n");
					}
					else
					{
						memcpy(buf, cn.bstrVal, SysStringLen(cn.bstrVal)*sizeof(WCHAR));
						wprintf(buf);
						wprintf(L"\n");						
					}				
				}
				VariantClear(&cn);
			}
		}
		VariantClear(&vtProp);
	}

	return WBEM_S_NO_ERROR;

//	return (0);
}


static HRESULT STDMETHODCALLTYPE SetStatus(IMyEventSink *this, LONG lFlags, HRESULT hResult, BSTR strParam, IWbemClassObject *pObjParam)
{
	int i = 0;

	return (0);
}

static IMyEventSinkVtbl IMyEventSink_Vtbl = { QueryInterface, AddRef, Release, Indicate, SetStatus };


IMyEventSink *CreateInstance()
{
	IMyEventSink *pMyEventSink = (IMyEventSink*)malloc(sizeof(IMyEventSink));
	pMyEventSink->lpVtbl = &IMyEventSink_Vtbl;
	pMyEventSink->count = 0;
	SecureZeroMemory(pMyEventSink->buf, sizeof(pMyEventSink->buf));

	pMyEventSink->lpVtbl->AddRef(pMyEventSink);

	return pMyEventSink;
}